package sk.fiit.jim;

import java.util.HashMap;
import java.util.Map;

import sk.fiit.jim.code_review.ReviewOk;

/**
 *  Settings.java
 *  
 *  Encapsulates global settings that alter the behaviour of the code throughout
 *  the entire project. Can change default settings for the game. Usually is
 *  meant to be set in ./scripts/config/settings.rb.
 *
 *@Title	Jim
 *@author	marosurbanec
 *@author	Androids
 */
@ReviewOk
public final class Settings{
	
	private Settings(){}
	
	private static Map<String, Object> settings = new HashMap<String, Object>();
	
	static{
		setDefaults();
	}

	/*
	 * Here are the default settings for the game. If not set differently in
	 * ./scripts/config/settings.rb, they will be read from here.
	 */
	private static void setDefaults(){
		settings.put("kalmanUseFilter", true);
		settings.put("kalmanDefaultQ", .475);
		settings.put("kalmanDefaultR", .375);
		settings.put("runGcOnPhaseStart", true);
		settings.put("runGui", true);
		settings.put("gravityAcceleration", 9.81);
		settings.put("maximumAngularChangePerQuantum", 7.0);
		settings.put("ignoreAccelerometer", false);
		settings.put("runTFTPserver", false);
	}
	
	public static void setValue(String key, Object value){
		settings.put(key, value);
	}
	
	public static double getDouble(String key){
		return (Double)get(key);
	}
	
	public static boolean getBoolean(String key){
		return (Boolean)get(key);
	}

	private static Object get(String key){
		Object setting = settings.get(key);
		if (setting == null)
			throw new IllegalStateException("Setting: " + key + " not found");
		return setting;
	}
}