package sk.fiit.jim.agent;

import sk.fiit.jim.Settings;
import sk.fiit.jim.agent.models.EnvironmentModel;
import sk.fiit.jim.init.Script;
import sk.fiit.jim.log.Log;
import sk.fiit.jim.log.LogType;

/**
 *  Planner.java
 *  
 *  Class implements planning module. Can plan several moves in a row.
 *
 *@Title	Jim
 *@author	marosurbanec
 *@author	Androids
 */
public class Planner{
	private static final double ENOUGH_TIME_FOR_PHASE = 200.0;
	private static final int ENOUGH_TIME = 5*1000*1000; //5 millisecond = 5,000,000 nanoseconds
	static Script planningScript = Script.createScript("Plan.instance.control");
	public static double nextTimePhaseEnds;
	public static long loopStart;

	//added by Androids
	private static String moveNames[] = {"turnright90", "walk_fine_fast2", "walkback2", "kick_right"};
	private static long count = 0;
	
	/*
	 * Method executes several moves in a row. Used to fill the queue in
	 * ./scripts/plan/plan.rb or to implement decision tree algorithm. Added
	 * by Androids
	 */
	public static void proceed(){
		planningScript.execute();
		if ((count > 1000) && (count < 1300)){
			moveNames = new String[4];
			moveNames[0] = "turnright90";
			moveNames[1] = "walk_fine_fast2";
			moveNames[2] = "walkback2";
			moveNames[3] = "kick_right";
		}
	}
	
	/*
	 * Returns the move names, so the plan queue can be filled. Added by
	 * Androids.
	 */
	public static String[] getPlanedMoves(){
		return moveNames;
	}
	
	//added by Androids
	public static void setPlanedMoves(String names[]){
		moveNames = names;
	}
	
	//added by Androids
	public static void clearPlanedMoves(){
		moveNames = null;
	}
	
	public static void runGarbageCollectIfEnoughTime(){
		if (!Settings.getBoolean("runGcOnPhaseStart"))
			return;
		long now = System.nanoTime();
		double serverTime = EnvironmentModel.SIMULATION_TIME;
		Log.debug(LogType.PLANNING, "Loop took %d ms", (now - loopStart));
		if ((now - loopStart) < ENOUGH_TIME && (nextTimePhaseEnds - serverTime) > ENOUGH_TIME_FOR_PHASE){
			Log.debug(LogType.PLANNING, "We have enough time, running GC and finalization");
			runGc();
		}
	}

	private static void runGc(){
		new Thread(new Runnable(){
			public void run(){
				System.runFinalization();
				System.gc();
			}
		}).start();
	}
}