package sk.fiit.jim.agent.models;

import java.util.Map.Entry;

import static sk.fiit.jim.log.LogType.AGENT_MODEL;
import sk.fiit.jim.agent.parsing.ParsedData;
import sk.fiit.jim.log.Log;
import sk.fiit.jim.math.Vector3D;

/**
 *  AgentPositionCalculator.java
 *  
 *  Calculates an approximation of agent's current position, based on the
 *  flags seen. Class assumes that agent's rotations are correctly calculated.
 *  
 *  Calculating is done by normalizing and reversing a given flag position.
 *  When this percieved relative position is added to the known position
 *  of the flag, a correct approximation of our position should be calculated.
 *
 *@Title	Jim
 *@author	marosurbanec
 *@author	Androids
 */
public class AgentPositionCalculator{
	private final AgentModel agent;

	public AgentPositionCalculator(AgentModel agent){
		this.agent = agent;
	}
	
	public void updatePosition(ParsedData data){
		if(data.fixedObjects == null || data.fixedObjects.size() < 1) {
			return;
		}
		
		Vector3D accumulator = Vector3D.cartesian(0.0, 0.0, 0.0);
		
		for(Entry<FixedObject, Vector3D> flag : data.fixedObjects.entrySet()){
			Vector3D absolute = flag.getKey().getAbsolutePosition();
			Vector3D seen = flag.getValue();
			Vector3D ourPosition = seen.rotateOverX(agent.rotationX).rotateOverY(agent.rotationY).rotateOverZ(agent.rotationZ).negate();
			ourPosition = ourPosition.add(absolute);
			accumulator = accumulator.add(ourPosition);
		}
		agent.position = accumulator.divide(data.fixedObjects.size());
		Log.log(AGENT_MODEL, "My position: [%.2f,%.2f,%.2f]", agent.position.getX(), agent.position.getY(), agent.position.getZ());
	}
}