package sk.fiit.jim.agent.models;

import sk.fiit.jim.math.Vector3D;

/**
 *  DynamicObject.java
 *  
 *  Class determines the position of a dynamic object, usually a ball.
 *  TODO: implement calculation of opponent position.
 *
 *@Title	Jim
 *@author	marosurbanec
 *@author	Androids
 */
public class DynamicObject {
	private static double LOW_TIME = 0.1;
	private Vector3D position = Vector3D.ZERO_VECTOR;
	private Vector3D relativePosition = Vector3D.ZERO_VECTOR;
	private Vector3D speed = Vector3D.ZERO_VECTOR;
	private Vector3D relativeSpeed = Vector3D.ZERO_VECTOR;
	private double lastTimeSeen = 0.0;

	//added by Androids
	public void setRelativePosition(Vector3D relativePos, double timeSeen){
		double elapsedTime = timeSeen - lastTimeSeen;
		if (this.relativePosition != Vector3D.ZERO_VECTOR)
			relativeSpeed = relativePos.subtract(this.relativePosition).divide(elapsedTime);
		this.relativePosition = relativePos;
	}
	
	public void setPosition(Vector3D absolutePosition, double timeSeen){
		double elapsedTime = timeSeen - lastTimeSeen;
		if (this.position != null)
			speed = absolutePosition.subtract(this.position).divide(elapsedTime);
		this.lastTimeSeen = timeSeen;
		this.position = absolutePosition;
	}
	
	public Vector3D getPosition(){
		return position;
	}

	//added by Androids
	public Vector3D getRelativePosition(){
		if (notSeenLong())
			return AgentModel.getInstance().relativize(position);
		return relativePosition;
	}
	
	//added by Androids
	private boolean notSeenLong() {
		return EnvironmentModel.SIMULATION_TIME - lastTimeSeen > LOW_TIME;
	}
	
	//added by Androids
	//commented by Juraj Belanji - not sure it works
/*	public Vector3D predictPositionIn(double offset){
		if (offset < 0)
			throw new IllegalArgumentException(String.format("Negative offset supplied: %.2f", offset));
		return speed.multiply(offset).add(position);
	}
	
	public Vector3D predictRelativePositionIn(double offset){
		AgentModel agentModel = AgentModel.getInstance();
		
		Vector3D withUnsyncedRotations = predictPositionIn(offset).
			rotateOverX(-agentModel.getRotationX()).
			rotateOverY(-agentModel.getRotationY()).
			rotateOverZ(-agentModel.getRotationZ());
		return withUnsyncedRotations.subtract(agentModel.getPosition());
	}*/
	
	public double getLastTimeSeen(){
		return lastTimeSeen;
	}
	
	public Vector3D getSpeed() {
		return speed;
	}
	
	public Vector3D getRelativeSpeed() {
		return relativeSpeed;
	}
	
	@Override
	public String toString(){
		return new StringBuilder().
			append("Absolute: ").
			append(position).
			append('\n').
			append("Relative: ").
			append(relativePosition).
			toString();
	}
}