package sk.fiit.jim.agent.models;

import java.util.HashMap;
import java.util.Map;

import sk.fiit.jim.agent.models.EnvironmentModel.Version;
import sk.fiit.jim.math.Vector3D;

/**
 *  FixedObjects.java
 *  Flags seen by SEE perceptor of the robot. Each flag keeps information
 *  about its global position. For proper functioning of this class,
 *  make sure {@link EnvironmentModel}.{@link Version} is properly set.
 *	
 *@Title	Jim
 *@author	marosurbanec
 *@author	Androids
 */
public enum FixedObject{
	OUR_LOWER_CORNER,
	OUR_UPPER_CORNER,
	OUR_UPPER_POST,
	OUR_LOWER_POST,
	THEIR_UPPER_CORNER,
	THEIR_LOWER_CORNER,
	THEIR_UPPER_POST,
	THEIR_LOWER_POST;
	
	/*
	 * The playground size is the same for 0.6.3 and 0.6.4 versions.
	 */
	public Vector3D getAbsolutePosition(){
		Map<FixedObject, Vector3D> mapping = EnvironmentModel.version == Version.VERSION_0_6_2 ? positions_0_6_2 : positions_0_6_3;
		return mapping.get(this);
	}
	
	@SuppressWarnings("serial")
	private static Map<FixedObject, Vector3D> positions_0_6_3 = new HashMap<FixedObject, Vector3D>(){{
		put(OUR_LOWER_CORNER, Vector3D.cartesian(-9.0, -6.0, 0));
		put(OUR_UPPER_CORNER, Vector3D.cartesian(-9.0, 6.0, 0));
		put(OUR_UPPER_POST, Vector3D.cartesian(-9.0, 1.0, 0.8));
		put(OUR_LOWER_POST, Vector3D.cartesian(-9.0, -1.0, 0.8));
		
		put(THEIR_LOWER_CORNER, Vector3D.cartesian(9.0, -6.0, 0));
		put(THEIR_UPPER_CORNER, Vector3D.cartesian(9.0, 6.0, 0));
		put(THEIR_UPPER_POST, Vector3D.cartesian(9.0, 1.0, 0.8));
		put(THEIR_LOWER_POST, Vector3D.cartesian(9.0, -1.0, 0.8));
	}};
	
	//TODO: for future reference 0.6.2 version will not be used. This part can be deleted.
	@SuppressWarnings("serial")
	private static Map<FixedObject, Vector3D> positions_0_6_2 = new HashMap<FixedObject, Vector3D>(){{
		put(OUR_LOWER_CORNER, Vector3D.cartesian(-6.0, -4.0, 0));
		put(OUR_UPPER_CORNER, Vector3D.cartesian(-6.0, 4.0, 0));
		put(OUR_UPPER_POST, Vector3D.cartesian(-6.0, 0.8, 0.75));
		put(OUR_LOWER_POST, Vector3D.cartesian(-6.0, -0.8, 0.75));
		
		put(THEIR_LOWER_CORNER, Vector3D.cartesian(6.0, -4.0, 0));
		put(THEIR_UPPER_CORNER, Vector3D.cartesian(6.0, 4.0, 0));
		put(THEIR_UPPER_POST, Vector3D.cartesian(6.0, 0.8, 0.75));
		put(THEIR_LOWER_POST, Vector3D.cartesian(6.0, -0.8, 0.75));
	}};
	
	@SuppressWarnings("serial")
	private static Map<String, FixedObject> namesInServerMessages = new HashMap<String, FixedObject>(){{
		put("G2R", THEIR_LOWER_POST);
		put("G1R", THEIR_UPPER_POST);
		put("F1R", THEIR_UPPER_CORNER);
		put("F2R", THEIR_LOWER_CORNER);
		
		put("G2L", OUR_LOWER_POST);
		put("G1L", OUR_UPPER_POST);
		put("F1L", OUR_UPPER_CORNER);
		put("F2L", OUR_LOWER_CORNER);
	}};
	
	public static FixedObject fromServerId(String id){
		return namesInServerMessages.get(id.toUpperCase());
	}
}