package sk.fiit.jim.agent.models.prediction;

import sk.fiit.jim.agent.models.AgentModel;
import sk.fiit.jim.agent.models.DynamicObject;
import sk.fiit.jim.agent.models.EnvironmentModel;
import sk.fiit.jim.agent.models.WorldModel;
import sk.fiit.jim.code_review.Problem;
import sk.fiit.jim.math.Vector3D;

/**
 *  Prophet.java
 *  
 *  Calculates the most probable outcome of events in a given time.
 *  TODO: implement the prediction module correctly - not sure it works.
 *
 *@Title	Jim
 *@author	marosurbanec
 *@author	Androids
 */
public class Prophet {
	private final WorldModel world;
	private Prophecy prophecy;
	private final double offset;
	private final double now;
	private AgentModel agent;
	
	public Prophet(double offset){
		this(WorldModel.getInstance(), AgentModel.getInstance(), offset, EnvironmentModel.SIMULATION_TIME);
	}

	public Prophet(WorldModel world, AgentModel agent, double offset, double now){
		this.world = world;
		this.agent = agent;
		this.offset = offset;
		this.now = now;
		prophecy = new Prophecy();
	}

	public Prophecy foretell(){
		calculateBallPosition();
		return prophecy;
	}

	@Problem("do not ignore friction")
	private void calculateBallPosition(){
		DynamicObject ball = world.getBall();
		if (ball.getSpeed() == null){
			prophecy.setBallRelativePosition(agent.relativize(Vector3D.ZERO_VECTOR));
			prophecy.setBallPosition(Vector3D.ZERO_VECTOR);
			prophecy.setBallPositionRelativized(agent.relativize(Vector3D.ZERO_VECTOR));
			return;
		}
		
		Vector3D whereItIsNow = ball.getSpeed().multiply(now - ball.getLastTimeSeen()).add(ball.getPosition());
		Vector3D whereWillItBe = ball.getSpeed().multiply(offset).add(whereItIsNow);
		Vector3D whereWillItBeBasedOnRelativeSpeed = ball.getRelativeSpeed().multiply(now - ball.getLastTimeSeen() + offset).add(ball.getRelativePosition());
		
		prophecy.setBallPosition(whereWillItBe);
		prophecy.setBallPositionRelativized(agent.relativize(whereWillItBe));
		prophecy.setBallRelativePosition(whereWillItBeBasedOnRelativeSpeed);
	}
}