package sk.fiit.jim.agent.moves;

import java.util.ArrayList;
import java.util.List;

/**
 *  Phase.java
 *  
 *  A structure encompassing data of a movement phase in a sequence of phases.
 *  Together with other phases it creates a whole LOW SKILL. Thus,
 *  LOW SKILL => PHASE relation is of type 1:n.
 *  
 *  Data in the structure:
 *  1. name - of the phase - has to be UNIQUE
 *  2. next - next phase in the sequence of phases. May be ommitted in case isFinal is set to true
 *  3. EFFECTORS - list of elementary joint moves executed in PARALLEL
 *  4. duration - time in miliseconds that the move should take. Rounded to the nearest multiple of 20ms (one time quantum on server)
 *  5. isFinal - denotes whether this phase is the last phase of a CYCLE. If so, the execution of a cycle may be broken
 *  6. finalizationPhase - valid only if isFinal == true. Works as next attribute in case a finalization is requested
 *  7. skipIfFlag - skips the execution of this phase if a such a flag is set to true
 *  8. setTrueFlag - LowSkill sets a flag to true AFTER successfully executing this phase
 *  9. setFalseFlag - LowSkill sets a flag to false PRIOR to executing this phase
 *
 *@Title	Jim
 *@author	marosurbanec
 */
public class Phase{
	public String name;
	public String next;
	public List<EffectorData> effectors = new ArrayList<EffectorData>();
	public double duration = 0.0;
	public String finalizationPhase;
	public boolean isFinal;
	public SkipFlag skipIfFlag;
	public SkipFlag setFlagTrue;
	public SkipFlag setFlagFalse;
	
	public Phase(){}
	public Phase(String name, double duration){
		this.name = name;
		this.duration = duration;
	}

	public static final Phase EMPTY_PHASE = new Phase(){{
		isFinal = true;
		name = "EMPTY_PHASE";
		next = "EMPTY_PHASE";
		finalizationPhase = "EMPTY_PHASE"; 
		//we can't set 0.0, that would cause ZeroDivisionException 
		duration = 20.0;
	}};
	
	@Override
	public String toString(){
		return name;
	}
}