package sk.fiit.jim.init;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import sk.fiit.jim.agent.AgentInfo;
import sk.fiit.jim.agent.communication.CommunicationThread;
import sk.fiit.jim.agent.server.TFTPServer;
import sk.fiit.jim.agent.server.TFTPServer.ServerMode;
import sk.fiit.jim.code_review.UnderConstruction;

/**
 * TestFrameworkMain.java
 * 
 * @Title	Jim
 * @author	Androids
 */
@UnderConstruction(todo = { "wrap communication into thread to achieve restart functionality" })
public class TestFrameworkMain {

	/**
	 * <code>TestFrameworkMain</code> is entry point which is called by Test
	 * Framework. This is especially in need when using machine learning for
	 * specifying
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		// Needed args
		// [1] - TFTP Port
		// [2] - Uniform number
		// [3] -
		// [4] -

		if (args.length != 2) {
			throw new IllegalArgumentException(
					"Bad number of arguments. See source code for "
							+ "better understanding of this error.");
		}

		// argument processing
		final int tftpPort = Integer.valueOf(args[0]).intValue();
		int uniformNumber = Integer.valueOf(args[1]).intValue();

		System.out.println("TFTP port: " + tftpPort + ", Uniform: "
				+ uniformNumber);

		new SkillsFromXmlLoader(new File("./moves2")).load();
		ScriptBoot.boot();
		// Configuration through command line override
		
		ExecutorService es = Executors.newFixedThreadPool(2);

		// Run TFTP server
		es.submit(new Runnable() {

			@Override
			public void run() {
				try {
					new TFTPServer(new File("./scripts/testframework"),
							new File("./scripts/testframework"), tftpPort,
							ServerMode.PUT_ONLY, System.out, System.err).run();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});

		// dependency injection is set in dependencies.rb => no need to do them
		// manually
		System.runFinalization();
		System.gc();

		AgentInfo.playerId = uniformNumber;
		
		es.submit(new CommunicationThread());
	}
}